type VideoPlayerEvents =
	'playing'
	| 'pause'
	| 'ended'
	| 'timeupdate'
	| 'error'
	| 'canplay'
	| 'durationchange'
	| 'progress'

type PlayerOptions = {
	enableLog?: boolean;
	className?: string;
}

enum PlayState {
	playing = 'playing',
	pause = 'pause',
	ended = 'ended',
	error = 'error',
}

export class VideoPlayer {
	private video?: HTMLVideoElement;
	private State: PlayState = PlayState.pause;
	private enable_log = true;
	private currentDuration = 0;


	newInstance(options?: PlayerOptions) {
		const { className, enableLog=true } = options || {};
		this.enable_log = enableLog || false;

		if (this.video) {
			this.video.remove();
		}
		if (this.video) {
			this.video.pause();
		}
		// Create video element
		const playerVideo = document.createElement('video');
		const playerId = `player-container-${Date.now().toString(16)}`;
		playerVideo.setAttribute('id', playerId);
		playerVideo.setAttribute('preload', 'auto');
		playerVideo.setAttribute('playsInline', 'true');
		playerVideo.setAttribute('webkit-playsinline', 'true');
		if (className) playerVideo.setAttribute('className', className);
		playerVideo.classList.add('digital-video-player');
		playerVideo.addEventListener('durationchange', (e) => {
			this.currentDuration = Math.floor(this.video?.duration || 0);
			this.log('video duration change',e)
		});
		playerVideo.addEventListener('error', (e) => {
			this.log('video error:',e)
		});
		this.video = playerVideo;
		return this;
	}

	/**
	 * 日志
	 * @param args
	 */
	log(...args: any[]) {
		if (this.enable_log) {
			console.log(...args);
		}
	}

	getVideo() {
		return this.video!;
	}

	// 暂停视频
	pause() {
		this.video?.pause();
		// 更新状态
		this.State = PlayState.pause;
	}

	/**
	 * 监听视频事件
	 * @param event
	 * @param callback
	 */
	on(event: VideoPlayerEvents, callback: () => void) {
		this.video?.addEventListener(event, callback);

	}

	off(event: VideoPlayerEvents, callback: () => void) {
		this.video?.removeEventListener(event, callback);
	}

	/**
	 * 获取或这是当前播放时间点
	 * @param time
	 */
	currentTime(time?: number) {
		if (!this.video) return 0;
		if (typeof time === 'number' && this.video.currentTime !== time) {
			try {
				this.video.currentTime = time;
			} catch (err) {
				this.log('Failed to set currentTime', err);
			}
		}
		return this.video.currentTime || 0;
	}

	src(url: string) {
		if (!this.video) return;
		this.video.src = url;
		this.video.load();

	}

	/**
	 * 播放
	 */
	play() {
		if (!this.video) return;
		this.video.play().then(() => {
			this.State = PlayState.playing;
		}).catch((err) => {
			this.State = PlayState.error;
			this.log('play error', err);
		});
	}

	duration() {
		return this.currentDuration || this.video?.duration || 0;
	}
	dispose(){
		if(!this.video){
			return;
		}
		// 暂停
		this.pause();
		// 移除
		this.video.parentNode?.removeChild(this.video);
		this.video = undefined;
	}
}

/**
 * player.on('pause', () => {
 * 			setState({playing: false, end: false, error: false})
 * 		})
 * 		player.on('playing', () => {
 * 			setState({playing: true, end: false, error: false})
 * 		})
 * 		player.on('ended', () => {
 * 			setState({end: true, playing: false, error: false})
 * 		})
 * 		player.on('timeupdate', () => {
 * 			props.onProgress?.(player.currentTime(), player.duration())
 * 		})
 * 		player.on('error', () => {
 * 			setState({end: false, playing: false, error: true})
 * 		})
 */

const videoPlayer = new VideoPlayer();
export default videoPlayer;