import {useSortable} from "@dnd-kit/sortable";
import {useSetState} from "ahooks";
import React, {useEffect} from "react";
import {Checkbox, Popconfirm} from "antd";

import ImageCover from '@/assets/images/cover.png'
import {
	IconDelete, IconDownloadOutline,
	IconEdit,
	IconGenerateFailed,
	IconGenerating,
	IconPlaying, IconRegenerate, IconRollbackCircle,
	IconWarningCircle
} from "@/components/icons";
import {VideoStatus} from "@/service/api/video.ts";
import {formatTime} from "@/util/strings.ts";
import {useTranslation} from "react-i18next";
import {saveAs} from "file-saver";
import {DeleteItemPopoverConfirm} from "@/components/message/confirm.tsx";
import {showLoading, showToast} from "@/components/message.ts";

type Props = {
	video: VideoInfo | LiveVideoInfo,
	additionOperationBefore?: React.ReactNode;
	additionOperationAfter?: React.ReactNode;
	editable?: boolean;
	downloadUrl?: string;
	sortable?: boolean;
	index?: number;
	checked?: boolean;
	active?: boolean;
	playing?: boolean;
	onCheckedChange?: (checked: boolean) => void;
	onPlay?: () => void;
	onEdit?: () => void;
	onRegenerate?: () => void;
	hideCheckBox?: boolean;
	onItemClick?: () => void;
	onRemove?: (action?: 'delete' | 'rollback') => void;
	removeIcon?: React.ReactNode;
	id: number;
	className?: string;
	type?: 'live' | 'create'
}

export const VideoListItem = (
	{
		id, video, onRemove, removeIcon, checked, playing,
		onCheckedChange, onEdit, active, editable, downloadUrl,
		className, sortable, type, index, onItemClick,
		additionOperationAfter, additionOperationBefore, onRegenerate, hideCheckBox
	}: Props) => {
	const {
		attributes, listeners,
		setNodeRef, transform
	} = useSortable({resizeObserverConfig: {}, id})

	const {t,i18n} = useTranslation()
	const [state, setState] = useSetState<{ checked?: boolean }>({})
	useEffect(() => {
		setState({checked})
	}, [checked])

	const generating = (type == 'create' && video.status == VideoStatus.Generating)
	const failed = (type == 'create' && (video.status != VideoStatus.Generating && video.status != VideoStatus.Generated))
	const handleDownloadVideo = () => {
		if (downloadUrl && video.status == VideoStatus.Generated) {
			const ext = downloadUrl.substring(downloadUrl.lastIndexOf('.'))
			const loading = showLoading(t('downloading'))
			try{
				saveAs(downloadUrl, `${video.title || video.video_title}${ext}`)
				loading.close()
			}catch (e){
				loading.update(t('download_failed'),'error')
			}

		}
	}
	return <div
		className={`video-item ${className}`}
		ref={setNodeRef} style={{transform: `translateY(${transform?.y || 0}px)`,}}
	>
		<div
			className={`list-row ${generating ? ' status-generating' : ''} ${failed ? 'status-generate-failed' : ''} ${active ? 'playing' : ''}`}>
			<div
				className="col number"
				{...(sortable && !generating ? listeners : {})}
				{...(sortable && !generating ? attributes : {})}
			>{index}</div>
			<div className="col cover cursor-pointer" onClick={onItemClick}>
				<div className="relative">
					<img className="w-[100px] h-[56px] object-cover border border-gray-200" src={video.cover || ImageCover}/>
					{generating &&
              <div
                  className={'absolute rounded inset-0 bg-black/40 backdrop-blur-[1px] text-white flex items-center justify-center'}>
                  <div className="text-center">
                      <IconGenerating className="inline-block text-xl"/>
                      <div className="text-xs">{t('video.generating')}</div>
                  </div>
              </div>
					}
					{failed &&
              <div
                  className={'absolute rounded inset-0 bg-black/40  backdrop-blur-[1px] text-white flex items-center justify-center'}>
                  <div className="text-center">
                      <IconGenerateFailed className="inline-block text-xl"/>
                      <div className="text-xs">{t('video.generate_failed')}</div>
                  </div>
              </div>
					}
					{/* && active*/}
					{!generating && !failed && playing && <div
              className={'absolute rounded inset-0 backdrop-blur-[1px] bg-black/40 text-white flex items-center justify-center'}>
              <div className="text-center">
                  <IconPlaying className="inline-block text-xl"/>
                  <div className="text-xs">{t('video.playing')}</div>
              </div>
          </div>}
				</div>
			</div>
			<div
				className="col title"
				{...(sortable && !generating ? listeners : {})}
				{...(sortable && !generating ? attributes : {})}
			>
				<div className="line-clamp-2">
					{video.title || video.video_title}
				</div>
			</div>
			<div
				className="col generated-time"
				{...(sortable && !generating ? listeners : {})}
				{...(sortable && !generating ? attributes : {})}
			>{video.ctime ? formatTime(video.ctime, 'min') : '-'}</div>
			<div className="col operation">
				{/*{sortable && !generating && (!active ?*/}
				{/*	<button className="hover:text-blue-500 cursor-move">*/}
				{/*		<MenuOutlined/>*/}
				{/*	</button> : <button disabled className="cursor-not-allowed"><MenuOutlined/></button>)}*/}
				<div className={"flex items-center justify-center gap-5"}>
					{downloadUrl && video.status == VideoStatus.Generated &&
              <button className="hover:text-blue-500" onClick={e => {
								e.preventDefault()
								e.stopPropagation()
								handleDownloadVideo?.()
							}} style={{fontSize: '1.1em'}} title={i18n.language == 'zh-CN'?'下载':'Download'}>
                  <IconDownloadOutline/>
              </button>}
					{additionOperationBefore}
					{editable && !generating && <>
						{onEdit && <button
                className="hover:text-blue-500" onClick={e => {
							e.preventDefault()
							e.stopPropagation()
							onEdit?.()
						}} style={{fontSize: '1.1em'}} title={i18n.language == 'zh-CN'?'修改':'Modify'}>
                <IconEdit/>
            </button>}
						{onRegenerate && <button
                className="text-red-400 hover:text-blue-500" onClick={e => {
							e.preventDefault()
							e.stopPropagation()
							onRegenerate?.()
						}} style={{fontSize: '1.1em'}} title={i18n.language == 'zh-CN'?'重新生成':'Regenerate'}>
                <IconRegenerate/>
            </button>}

						{onRemove && !failed && <DeleteItemPopoverConfirm
                description={failed ? t('video.rollback_confirm_title') : undefined}
                onConfirm={() => onRemove(failed ? 'rollback' : 'delete')}>
                <button className="hover:text-blue-500" title={
									failed ? (i18n.language == 'zh-CN'?'重新生成':'Regenerate') : i18n.language == 'zh-CN'?'删除':'Delete'
								} style={{fontSize:20}}>
									{removeIcon ? removeIcon : (failed ?
										<IconRollbackCircle/> :
										<IconDelete/>)}
                </button>
            </DeleteItemPopoverConfirm>}
						{hideCheckBox ? <></> :
							<Checkbox checked={state.checked} onChange={() => {
								if (onCheckedChange) {
									onCheckedChange(!state.checked)
								} else {
									setState({checked: !state.checked})
								}
							}}/>}
          </>}
					{additionOperationAfter}
				</div>
			</div>
		</div>
	</div>
}